from common import *


async def not_implemented(connection, rest):
    connection.response("502", ":P")
    return True


@aioftp_setup()
@with_connection
async def test_client_fallback_to_pasv_at_list(loop, client, server):
    server.epsv = not_implemented
    await client.login()
    await client.list()



import nose

import aioftp

from common import *  # noqa


def test_throttle_repr():

    t = aioftp.Throttle(loop="loop")
    nose.tools.eq_(repr(t), "Throttle(loop='loop', limit=None, reset_rate=10)")


def test_user_repr():

    u = aioftp.User()
    nose.tools.eq_(
        repr(u),
        "User("
            "None, "
            "None, "
            "base_path=PosixPath('.'), "
            "home_path=PurePosixPath('/'), "
            "permissions=["
                "Permission(PurePosixPath('/'), "
                "readable=True, "
                "writable=True)], "
            "maximum_connections=None, "
            "read_speed_limit=None, "
            "write_speed_limit=None, "
            "read_speed_limit_per_connection=None, "
            "write_speed_limit_per_connection=None)"
    )


def test_memory_path_io_repr():

    from aioftp.pathio import Node

    pio = aioftp.MemoryPathIO(loop="loop")
    pio.fs = [Node("dir", "/", 1, 1, content=[])]
    nose.tools.eq_(
        repr(pio),
        "[Node(type='dir', name='/', ctime=1, mtime=1, content=[])]"
    )


def test_permission_representation():

    p = aioftp.Permission(writable=False)
    nose.tools.eq_(
        repr(p),
        "Permission(PurePosixPath('/'), readable=True, writable=False)"
    )



import pathlib
import asyncio
import datetime
import itertools

import nose

import aioftp
from common import *  # noqa


def test_parse_directory_response():
    nose.tools.eq_(
        aioftp.Client.parse_directory_response('foo "baz "" test nop" """""fdfs """'),
        pathlib.PurePosixPath('baz " test nop'),
    )


def test_connection_del_future():
    loop = asyncio.new_event_loop()
    c = aioftp.Connection(loop=loop)
    c.foo = "bar"
    del c.future.foo


@nose.tools.raises(AttributeError)
def test_connection_not_in_storage():
    loop = asyncio.new_event_loop()
    c = aioftp.Connection(loop=loop)
    getattr(c, "foo")


@nose.tools.raises(ValueError)
def test_available_connections_too_much_acquires():
    ac = aioftp.AvailableConnections(3)
    ac.acquire()
    ac.acquire()
    ac.acquire()
    ac.acquire()


@nose.tools.raises(ValueError)
def test_available_connections_too_much_releases():
    ac = aioftp.AvailableConnections(3)
    ac.acquire()
    ac.release()
    ac.release()


def test_parse_pasv_response():
    p = aioftp.Client.parse_pasv_response
    nose.tools.eq_(p("(192,168,1,0,1,0)"), ("192.168.1.0", 256))


def test_parse_epsv_response():
    p = aioftp.Client.parse_epsv_response
    nose.tools.eq_(p("some text (ha-ha) (|||665|) ((((666() (|fd667s)."), (None, 666))
    nose.tools.eq_(p("some text (ha-ha) (|||665|) (6666666)."), (None, 666))


def _c_locale_time(d):
    with aioftp.common.setlocale("C"):
        return d.strftime("%b %d %H:%M")


def test_parse_list_datetime_not_older_than_6_month_format():
    p = aioftp.Client.parse_ls_date
    date_to_p = lambda d: d.strftime("%Y%m%d%H%M00")  # noqa
    dates = (
        datetime.datetime(year=2000, month=1, day=1),
        datetime.datetime(year=2000, month=12, day=31),
    )
    dt = datetime.timedelta(seconds=15778476 // 2)
    deltas = (datetime.timedelta(), dt, -dt)
    for now, delta in itertools.product(dates, deltas):
        d = now + delta
        nose.tools.eq_(p(_c_locale_time(d), now=d), date_to_p(d))



import nose

from common import *  # noqa
import aioftp


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_retr_0(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    with tmp_file.open(mode="w") as fout:

        fout.write("foobar")

    await client.login()
    async with client.download_stream("foo.txt", offset=0) as stream:

        r = await stream.read()

    await client.quit()

    tmp_file.unlink()
    nose.tools.eq_(r, b"foobar")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_retr_3(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    with tmp_file.open(mode="w") as fout:

        fout.write("foobar")

    await client.login()
    async with client.download_stream("foo.txt", offset=3) as stream:

        r = await stream.read()

    await client.quit()

    tmp_file.unlink()
    nose.tools.eq_(r, b"bar")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_retr_100(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    with tmp_file.open(mode="w") as fout:

        fout.write("foobar")

    await client.login()
    async with client.download_stream("foo.txt", offset=100) as stream:

        r = await stream.read()

    await client.quit()

    tmp_file.unlink()
    nose.tools.eq_(r, b"")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_stor_0(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    await client.login()
    async with client.upload_stream("foo.txt", offset=0) as stream:

        await stream.write(b"foobar")

    await client.quit()

    with tmp_file.open(mode="rb") as fin:

        r = fin.read()

    tmp_file.unlink()
    nose.tools.eq_(r, b"foobar")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_stor_3(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    with tmp_file.open(mode="w") as fout:

        fout.write("foobar")

    await client.login()
    async with client.upload_stream("foo.txt", offset=3) as stream:

        await stream.write(b"foo")

    await client.quit()

    with tmp_file.open(mode="rb") as fin:

        r = fin.read()

    tmp_file.unlink()
    nose.tools.eq_(r, b"foofoo")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_stor_10(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    with tmp_file.open(mode="w") as fout:

        fout.write("foobar")

    await client.login()
    async with client.upload_stream("foo.txt", offset=10) as stream:

        await stream.write(b"foo")

    await client.quit()

    with tmp_file.open(mode="rb") as fin:

        r = fin.read()

    tmp_file.unlink()
    nose.tools.eq_(r, b"foobar" + b"\x00" * 4 + b"foo")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_appe_10(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    with tmp_file.open(mode="w") as fout:

        fout.write("foobar")

    await client.login()
    async with client.append_stream("foo.txt", offset=10) as stream:

        await stream.write(b"foo")

    await client.quit()

    with tmp_file.open(mode="rb") as fin:

        r = fin.read()

    tmp_file.unlink()
    nose.tools.eq_(r, b"foobar" + b"\x00" * 4 + b"foo")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_restart_reset(loop, client, server, *, tmp_dir):

    tmp_file = tmp_dir / "foo.txt"
    with tmp_file.open(mode="w") as fout:

        fout.write("foobar")

    await client.login()
    await client.command("REST 3", "350")
    async with client.download_stream("foo.txt") as stream:

        r = await stream.read()

    await client.quit()

    tmp_file.unlink()
    nose.tools.eq_(r, b"foobar")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@expect_codes_in_exception("501")
@with_connection
async def test_restart_syntax_error(loop, client, server):

    await client.login()
    await client.command("REST 3abc", "350")



import pathlib

import nose

from common import *  # noqa


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_create_and_remove_directory(loop, client, server, *, tmp_dir):

    await client.login()
    await client.make_directory("bar")
    (path, stat), *_ = files = await client.list()
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("bar"))
    nose.tools.eq_(stat["type"], "dir")

    await client.remove_directory("bar")
    files = await client.list()
    nose.tools.eq_(len(files), 0)


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_create_and_remove_directory_long(loop, client, server, *,
                                                tmp_dir):

    await client.login()
    await client.make_directory("bar/baz")
    (path, stat), *_ = files = await client.list()
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("bar"))
    nose.tools.eq_(stat["type"], "dir")

    (path, stat), *_ = files = await client.list("bar")
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("bar/baz"))
    nose.tools.eq_(stat["type"], "dir")

    await client.remove_directory("bar/baz")
    await client.remove_directory("bar")
    files = await client.list()
    nose.tools.eq_(len(files), 0)


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_create_directory_long_no_parents(loop, client, server, *,
                                                tmp_dir):

    await client.login()
    await client.make_directory("bar/baz", parents=False)
    await client.remove_directory("bar/baz")
    await client.remove_directory("bar")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_change_directory(loop, client, server, *, tmp_dir):

    await client.login()
    await client.make_directory("bar")
    await client.change_directory("bar")
    cwd = await client.get_current_directory()
    nose.tools.eq_(cwd, pathlib.PurePosixPath("/bar"))

    await client.change_directory()
    cwd = await client.get_current_directory()
    nose.tools.eq_(cwd, pathlib.PurePosixPath("/"))

    await client.remove_directory("bar")
    files = await client.list()
    nose.tools.eq_(len(files), 0)


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@expect_codes_in_exception("550")
@with_tmp_dir("foo")
async def test_change_directory_not_exist(loop, client, server, *, tmp_dir):

    await client.login()
    await client.change_directory("bar")


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_rename_empty_directory(loop, client, server, *, tmp_dir):

    await client.login()
    await client.make_directory("bar")
    (path, stat), *_ = files = await client.list()
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("bar"))
    nose.tools.eq_(stat["type"], "dir")

    await client.rename("bar", "baz")
    (path, stat), *_ = files = await client.list()
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("baz"))
    nose.tools.eq_(stat["type"], "dir")

    await client.remove_directory("baz")
    files = await client.list()
    nose.tools.eq_(len(files), 0)


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests/foo"),)], {}))
@with_connection
@with_tmp_dir("foo")
async def test_rename_non_empty_directory(loop, client, server, *, tmp_dir):

    await client.login()
    await client.make_directory("bar")
    (path, stat), *_ = files = await client.list()
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("bar"))
    nose.tools.eq_(stat["type"], "dir")

    tmp_file = tmp_dir / "bar" / "foo.txt"
    tmp_file.touch()

    await client.make_directory("hurr")
    await client.rename("bar", "hurr/baz")
    (path, stat), *_ = files = await client.list()
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("hurr"))
    nose.tools.eq_(stat["type"], "dir")

    (path, stat), *_ = files = await client.list("hurr")
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("hurr/baz"))
    nose.tools.eq_(stat["type"], "dir")

    (path, stat), *_ = files = await client.list("/hurr/baz")
    nose.tools.eq_(len(files), 1)
    nose.tools.eq_(path, pathlib.PurePosixPath("/hurr/baz/foo.txt"))
    nose.tools.eq_(stat["type"], "file")

    tmp_file = tmp_dir / "hurr" / "baz" / "foo.txt"
    tmp_file.unlink()
    await client.remove_directory("hurr/baz")
    await client.remove_directory("hurr")
    files = await client.list()
    nose.tools.eq_(len(files), 0)


class FakeErrorPathIO(aioftp.PathIO):

    def list(self, path):

        class Lister(aioftp.AbstractAsyncLister):
            iter = None

            @aioftp.pathio.universal_exception
            @aioftp.with_timeout
            async def __anext__(self):
                if self.iter is None:
                    self.iter = path.glob("*")
                try:
                    raise Exception("KERNEL PANIC")
                except StopIteration:
                    raise StopAsyncIteration

        return Lister(timeout=self.timeout, loop=self.loop)


@aioftp_setup(
    server_args=([(aioftp.User(base_path="tests"),)],
                 {"path_io_factory": FakeErrorPathIO}))
@expect_codes_in_exception("451")
@with_connection
async def test_exception_in_list(loop, client, server):

    await client.login()
    await client.list()



import asyncio
import functools
import logging
import pathlib
import shutil
import socket
import ssl

import nose
import trustme

import aioftp


ca = trustme.CA()
server_cert = ca.issue_server_cert("127.0.0.1", "::1")

ssl_server = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
server_cert.configure_cert(ssl_server)

ssl_client = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
ca.configure_trust(ssl_client)

PORT = 8888


@nose.tools.nottest
def aioftp_setup(*, server_args=([], {}), client_args=([], {})):

    def decorator(f):

        @functools.wraps(f)
        def wrapper():
            s_args, s_kwargs = server_args
            c_args, c_kwargs = client_args

            def run_in_loop(s_args, s_kwargs, c_args, c_kwargs, s_ssl=None, c_ssl=None):
                logging.basicConfig(
                    level=logging.INFO,
                    format="%(asctime)s [%(name)s] %(message)s",
                    datefmt="[%H:%M:%S]:",
                )
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(None)
                server = aioftp.Server(*s_args, loop=loop, ssl=s_ssl, **s_kwargs)
                client = aioftp.Client(*c_args, loop=loop, ssl=c_ssl, **c_kwargs)
                try:
                    loop.run_until_complete(f(loop, client, server))
                finally:
                    if hasattr(server, "server"):
                        loop.run_until_complete(server.close())
                    if hasattr(client, "writer"):
                        client.close()
                    loop.close()

            if "path_io_factory" not in s_kwargs:
                for factory in (aioftp.PathIO, aioftp.AsyncPathIO):
                    s_kwargs["path_io_factory"] = factory
                    run_in_loop(s_args, s_kwargs, c_args, c_kwargs)
                    run_in_loop(s_args, s_kwargs, c_args, c_kwargs, ssl_server, ssl_client)
            else:
                run_in_loop(s_args, s_kwargs, c_args, c_kwargs)
                run_in_loop(s_args, s_kwargs, c_args, c_kwargs, ssl_server, ssl_client)

        return wrapper

    return decorator


@nose.tools.nottest
def with_connection(f=None, *, host="127.0.0.1", port=PORT,
                    family=socket.AF_INET):

    def decorator(f):

        @functools.wraps(f)
        async def wrapper(loop, client, server):
            try:
                await server.start(host, port, family=family)
                await client.connect(host, port)
                await f(loop, client, server)
            finally:
                client.close()
                await server.close()

        return wrapper
    if f is None:
        return decorator
    else:
        return decorator(f)


@nose.tools.nottest
def expect_codes_in_exception(*codes):

    def decorator(f):

        @functools.wraps(f)
        async def wrapper(*args):
            try:
                await f(*args)
            except aioftp.StatusCodeError as exc:
                nose.tools.eq_(exc.received_codes, codes)
            else:
                raise Exception("There was no exception")

        return wrapper
    return decorator


@nose.tools.nottest
def with_tmp_dir(name):

    def decorator(f):

        @functools.wraps(f)
        async def wrapper(*args):
            tmp_dir = pathlib.Path("tests") / name
            tmp_dir.mkdir(parents=True)
            try:
                await f(*args, tmp_dir=tmp_dir)
            finally:
                shutil.rmtree(str(tmp_dir))

        return wrapper
    return decorator



import re
import pathlib
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


BASE_PATH = pathlib.Path(__file__).parent
try:
    version = re.findall(r"""^__version__ = "([^']+)"\r?$""",
                         (BASE_PATH / "aioftp" / "__init__.py").read_text(),
                         re.M)[0]
except IndexError:
    raise RuntimeError("Unable to determine version.")


class NoseTestCommand(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pathlib
        import shutil
        import nose

        for path in pathlib.Path("tests").glob("*"):
            if path.is_dir():
                shutil.rmtree(str(path))
        cov = pathlib.Path(".coverage")
        if cov.exists():
            cov.unlink()
        nose.main(argv=["nosetests"])


setup(
    name="aioftp",
    version=version,
    description=("ftp client/server for asyncio"),
    long_description=(BASE_PATH / "README.rst").read_text(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: File Transfer Protocol (FTP)",
    ],
    author="pohmelie",
    author_email="multisosnooley@gmail.com",
    url="https://github.com/aio-libs/aioftp",
    license="Apache 2",
    packages=find_packages(),
    python_requires=" >= 3.5.3",
    install_requires=[],
    tests_require=["nose", "coverage", "trustme"],
    cmdclass={"test": NoseTestCommand},
    include_package_data=True
)



import socket
import os

from common import *


if "TRAVIS" not in os.environ:
    @aioftp_setup()
    @with_connection(host="::1", family=socket.AF_INET6)
    async def test_ipv6_list(loop, client, server):
        await client.login()
        await client.list()
